local bit = require 'bit'
local ffi = require 'ffi'
local sample_clock = require 'sample_clock'

local bit_band = bit.band
local bit_bor = bit.bor
local bit_rshift = bit.rshift
local ffi_cast = ffi.cast
local ffi_copy = ffi.copy
local ffi_new = ffi.new
local ffi_sizeof = ffi.sizeof
local ffi_string = ffi.string

local htonl = ffi.abi('le') and bit.bswap or bit.tobit

ffi.cdef[[
typedef struct __attribute((packed)) message_float64 {
    int64_t     timestamp[1];
    double      value[1];
} message_float64;
]]

local c_type = 'message_float64'

local M = {}

function M.is_heartbeat(msg)
    return isnan(msg.value[0])
end

local NaN = 0/0

local function isnan(x)
    return x ~= x
end

function M.set_heartbeat(msg, setting)
    msg.value[0] = setting and NaN or msg.value[0]
end

function M.dump(msg)
    return string.format('%.9f: %f',
        sample_clock.secs(self.timestamp[0]), tonumber(self.value[0]))
end

function M.__tostring(msg)
    return string.format('%f', tonumber(self.value[0]))
end

M.__index = M

local function init(self)
    local msg = self.new({0})
    return msg
end

return {
    new = ffi.metatype(c_type, M),
    c_type_name = c_type,
    c_type = ffi.typeof(c_type),
    ptr_type = ffi.typeof(c_type .. '*'),
    size = ffi.sizeof(c_type),
    init = init,
}
