--! \file

local common = require 'mx.device_common'
local underscore = require 'underscore'

local m = {}

--[[
Example of the setup that will be used this these channel types
{
    connector = '@connector1.mx878b-AABBCC',
    input_channel = '@connector3.mx1601b-AABBCC',
    calibration = {
        {
            purpose = "output_scaling",
            mode = "defined_slope_intercept",
            slope = 1.0,
            intercept = 0.0
        },
    },
    digital_filter_type = 'none',
    digital_filter_attenuation_frequency = 0,
}


This is the XML template retreived from the device.  It is here for
documentation purposes.

<Connector address="1">
    <Mode choice="Off">
        <Off />
        <AnalogOut>
            <Mode choice="Voltage">
                <Voltage />
            </Mode>
            <Identification>
                <ConnectorId></ConnectorId>
                <ConnectorName></ConnectorName>
                <CalDate></CalDate>
                <CalInitials> </CalInitials>
                <CalPeriod>0</CalPeriod>
            </Identification>
            <Input address="1">
                <ModuleReference>UUID=001234</ModuleReference>
                <SignalReference>AnalogIn_Connector1.Signal1</SignalReference>
                <Description>analog output: input source</Description>
            </Input>
            <Scaling>
                <Scalingtype choice="LinTable">
                    <LinTable>
                        <Count>2</Count>
                        <Point address="1">
                            <X>0.000000</X>
                            <Y>0.000000</Y>
                        </Point>
                        <Point address="2">
                            <X>10.000000</X>
                            <Y>10.000000</Y>
                        </Point>
                    </LinTable>
                    <Gain>
                        <m>1.000000</m>
                        <b>0.000000</b>
                    </Gain>
                </Scalingtype>
            </Scaling>
            <FilterType choice="Off">
                <Off />
                <Lowpass>
                    <FiltCharact>1</FiltCharact>
                    <FiltFreq>100.000000</FiltFreq>
                </Lowpass>
            </FilterType>
            <Output>
                <SignalReference>AnalogOut_Connector1.Signal1</SignalReference>
            </Output>
        </AnalogOut>
    </Mode>
</Connector>
]]

-- default XML for this channel type
local default_xml = [[<?xml version="1.0" encoding="UTF-8"?>
                <AdditionalConnector>
                    <Connector address="#conn_num#">
                        <Mode choice="Off">
                            </ Off>
                        </Mode>
                    </Connector#conn_num#>
                </AdditionalConnector>
]]

-- Function used to return the XML used to set the channel to it's default
-- configuration
function m:get_default_xml(channel_num)
    return default_xml:gsub('#conn_num#', tostring(channel_num))
end

m.json2xpath = {}

underscore.extend(m.json2xpath, common.json2xpath)

-- List of parameters in a channel setup that we do NOT want sent off to the MX
-- device
m.ignore_list = {}
underscore.extend(m.ignore_list, common.ignore_list)

-- List of parameters returned by MX device that we don't verify after setting
-- up a channel.
m.dont_verify_list = {}
underscore.extend(m.dont_verify_list, common.dont_verify_list)

-- Specification of the default settings of this channel type
m.default_spec = {
    calibration = {
        mode = 'defined_slope_intercept',
        slope = 1,
        intercept = 0,
    },
    digital_filter_type = 'none',
}

return m
