--[[
	Lua module for sorting translated sxr rows for GUI display
--]]


local s = {}

function s:run( groups_by_group_index ,rows)
	local data_by_position = {}

	for i , group in ipairs ( groups_by_group_index) do
		local has_parent = groups_by_group_index[ group.ParentIndex]
		if has_parent then
			self:store_group_by_parent( data_by_position ,group )
		else
			table.insert( data_by_position ,{})
			data_by_position[ #data_by_position].gi = group.GroupIndex
			data_by_position[ #data_by_position].pi = group.PositionIndex
		end
	end

	for i ,row in ipairs( rows) do
		self:store_row_by_group( data_by_position ,row)
	end

	data_by_position = { data_by_position[1] }

	return self:sort_data_by_position( data_by_position)
end

--========== Helpers =================
	function s:store_group_by_parent( current_level ,group )
		for i ,value in ipairs( current_level ) do
			if type( value) == 'table' then
				if value.gi == group.ParentIndex then
					table.insert( current_level[ i] ,{})
					local size = #current_level[ i]
					current_level[ i][ size].gi = group.GroupIndex
					current_level[ i][ size].pi = group.PositionIndex
				else
					self:store_group_by_parent( current_level[ i] , group)
				end
			end
		end
	end

	function s:store_row_by_group( current_level ,row)
		for i ,value in ipairs( current_level) do
			if type( value) == 'table' then
				if value.gi == row.group_index then
					table.insert( current_level[ i] ,row )
				else
					self:store_row_by_group( current_level[ i] ,row )
				end
			end
		end
	end

	function s:sort_data_by_position( array)
		array = self:merge_sort( array ,true )
		for i,v in ipairs( array) do
			if type(v) == 'table' and v.gi then
				array[i] = self:sort_data_by_position( array[i] )
			end
		end
		return array
	end

	function s:merge_sort( array ,root)
		local a ,b	--a and b are half-arrays. Names shortened for brevity.

		if #array <= 1 then 
			return array
		elseif #array == 2 then
			local dpa = array[1].position_index or array[1].pi
			local dpb = array[2].position_index or array[2].pi

			if dpa < dpb then
				return array
			else
				return { array[2] ,array[1] ,gi=array.gi ,pi=array.pi}
			end
		else
			a,b = self:split( array)
			a = self:merge_sort( a ,false)
			b = self:merge_sort( b ,false)
		end

		local result = {}
		local c = { a=1 ,b=1}
		for i=1,(#a+#b) do
			local dpa,dpb
			local value
			if c.a <= #a then
				dpa = a[c.a].position_index or a[c.a].pi
			else
				dpa = nil
			end
			if c.b <= #b then
				dpb = b[c.b].position_index or b[c.b].pi
			else
				dpb = nil
			end

			if not dpa then
				value = { b[c.b] ,'b'}
			elseif not dpb then
				value = { a[c.a] ,'a'}
			else
				if dpa > dpb then
					value = { b[c.b] ,'b'}
				else
					value = { a[c.a] ,'a'}
				end
			end

			table.insert( result ,value[1])
			c[ value[2]] = c[ value[2]] + 1
		end

		if root then
			result.gi = array.gi
			result.pi = array.pi
		end
		return result
	end

	function s:split( array ,index )
		local a ,b = {} ,{}
		if not index then
			index = math.floor(#array/2)
		end
		for i=1,index do
			table.insert( a ,array[i] )
		end
		for i=index+1,#array do
			table.insert( b ,array[i] )
		end

		return a,b
	end


return s