local m = {}

-- List of keys from the 'teds_sensor' section that will be checked against
-- the same keys in the base table.
local keys = {
    'calibration',
    'output_units',
    'sensor_input',
    'source_physical_units',
    'transducer_power'
}

-- Function use to deep compare two tables
local function tables_equal(t1, t2)
    for k, v in pairs(t1) do
        v = tonumber(v) or v
        t2[k] = tonumber(t2[k]) or t2[k]
        if not t2[k] then
            -- Entry does not exist in second table
            return
        elseif type(v) ~= type(t2[k]) then
            -- Entry types to no match
            return
        elseif type(v) == 'table' then
            -- Entry is a table
            return tables_equal(v, t2[k])
        elseif v ~= t2[k] then
            -- Entry values do not match
            return
        end
    end

    -- Everything matches
    return true
end

-- Function that returns true if the 'teds_sensor' section of a stanza differs
-- from the base fields of the stanza
function m.is_teds_edited(stanza)
    if not stanza.teds_sensor or stanza.teds_sensor.status == nil or
        stanza.teds_sensor.status ~= 'connected' then
        return
    end
    for _, key in ipairs(keys) do
        if stanza.teds_sensor[key] then
            stanza.teds_sensor[key] = tonumber(stanza.teds_sensor[key]) or stanza.teds_sensor[key]
            stanza.key = tonumber(stanza[key]) or stanza[key]
            if type(stanza.teds_sensor[key]) == 'table' then
                if not tables_equal(stanza.teds_sensor[key], stanza[key]) then
                    return true
                end
            elseif stanza.teds_sensor[key] ~= stanza[key] then
                return true
            end
        end
    end
end

-- Function used to find a stanza in a list of start channel entries by
-- reference.
function m.find_stanza_by_reference(start_channel_list, ref)
    local connector = string.format('^@connector%d.',
        ref:lower():match('connector(%d+)'))
    for _, entry in ipairs(start_channel_list) do
        if entry.spec and entry.spec.connector and
            entry.spec.connector:find(connector) then
            return entry
        end
    end
end

return m
